@@ -29,6 +29,7 @@ import tempfile
 import threading
 import time
 
+from distutils.version import LooseVersion
 from distutils.version import StrictVersion
 import docker
 import git
@@ -46,13 +47,13 @@ PROJECT_ROOT = os.path.abspath(os.path.join(
 if PROJECT_ROOT not in sys.path:
     sys.path.insert(0, PROJECT_ROOT)
 
-from kolla.common import config as common_config
-from kolla.common import task
-from kolla.common import utils
-from kolla import exception
-from kolla.template import filters as jinja_filters
-from kolla.template import methods as jinja_methods
-from kolla import version
+from kolla.common import config as common_config  # noqa
+from kolla.common import task  # noqa
+from kolla.common import utils  # noqa
+from kolla import exception  # noqa
+from kolla.template import filters as jinja_filters  # noqa
+from kolla.template import methods as jinja_methods  # noqa
+from kolla import version  # noqa
 
 
 LOG = utils.make_a_logger()
@@ -81,6 +82,7 @@ SKIPPED_IMAGES = {
         "almanach-base",
         "bifrost-base",
         "blazar-base",
+        "cyborg-base",
         "dragonflow-base",
         "freezer-base",
         "karbor-base",
@@ -96,10 +98,10 @@ SKIPPED_IMAGES = {
     ],
     'centos+source': [
         "ovsdpdk",
-        "searchlight-base",
+        "tripleoclient",
         # TODO(jeffrey4l): remove tripleo-ui when following bug is fixed
         # https://bugs.launchpad.net/tripleo/+bug/1744215
-        "tripleo-ui"
+        "tripleo-ui",
     ],
     'ubuntu+binary': [
         "almanach-base",
@@ -107,6 +109,8 @@ SKIPPED_IMAGES = {
         "blazar-base",
         "cloudkitty-base",
         "congress-base",
+        "crane",
+        "cyborg-base",
         "dragonflow-base",
         "ec2-api",
         "freezer-base",
@@ -125,6 +129,7 @@ SKIPPED_IMAGES = {
         "searchlight-base",
         "solum-base",
         "tacker-base",
+        "tripleoclient",
         "tripleo-ui",
         "vitrage-base",
         "vmtp",
@@ -132,9 +137,11 @@ SKIPPED_IMAGES = {
         "zun-base",
     ],
     'ubuntu+source': [
+        "crane",
+        "cyborg-base",
         # There is no qdrouterd package for ubuntu bionic
         "qdrouterd",
-        "searchlight-base",
+        "tripleoclient",
         # TODO(jeffrey4l): remove tripleo-ui when following bug is fixed
         # https://bugs.launchpad.net/tripleo/+bug/1744215
         "tripleo-ui"
@@ -145,6 +152,8 @@ SKIPPED_IMAGES = {
         "blazar-base",
         "cloudkitty-base",
         "congress-base",
+        "crane",
+        "cyborg-base",
         "dragonflow-base",
         "ec2-api",
         "freezer-base",
@@ -162,6 +171,7 @@ SKIPPED_IMAGES = {
         "sensu-base",
         "solum-base",
         "tacker-base",
+        "tripleoclient",
         "tripleo-ui",
         "vitrage-base",
         "vmtp",
@@ -169,14 +179,18 @@ SKIPPED_IMAGES = {
         "zun-base"
     ],
     'debian+source': [
+        "crane",
+        "cyborg-base",
         "sensu-base",
-        "searchlight-base",
+        "tripleoclient",
         "tripleo-ui"
     ],
     'oraclelinux+binary': [
         "almanach-base",
         "bifrost-base",
         "blazar-base",
+        "crane",
+        "cyborg-base",
         "dragonflow-base",
         "freezer-base",
         "karbor-base",
@@ -193,7 +207,7 @@ SKIPPED_IMAGES = {
     'oraclelinux+source': [
         "bifrost-base",
         "ovsdpdk",
-        "searchlight-base",
+        "tripleoclient",
         # TODO(jeffrey4l): remove tripleo-ui when following bug is fixed
         # https://bugs.launchpad.net/tripleo/+bug/1744215
         "tripleo-ui"
@@ -304,11 +318,6 @@ class PushIntoQueueTask(task.Task):
         self.success = True
 
 
-class PushError(Exception):
-    """Raised when there is a problem with pushing image to repository."""
-    pass
-
-
 class PushTask(DockerTask):
     """Task that pushes an image to a docker repository."""
 
@@ -332,9 +341,6 @@ class PushTask(DockerTask):
                                   ' have the correct privileges to run Docker'
                                   ' (root)')
             image.status = STATUS_CONNECTION_ERROR
-        except PushError as exception:
-            self.logger.error(exception)
-            image.status = STATUS_PUSH_ERROR
         except Exception:
             self.logger.exception('Unknown error when pushing')
             image.status = STATUS_PUSH_ERROR
@@ -359,10 +365,8 @@ class PushTask(DockerTask):
             if 'stream' in response:
                 self.logger.info(response['stream'])
             elif 'errorDetail' in response:
-                raise PushError(response['errorDetail']['message'])
-
-        # Reset any previous errors.
-        image.status = STATUS_BUILT
+                image.status = STATUS_ERROR
+                self.logger.error(response['errorDetail']['message'])
 
 
 class BuildTask(DockerTask):
@@ -682,6 +686,13 @@ class KollaWorker(object):
         self.install_type = conf.install_type
         self.tag = conf.tag
         self.base_arch = conf.base_arch
+        self.debian_arch = self.base_arch
+        if self.base_arch == 'aarch64':
+            self.debian_arch = 'arm64'
+        elif self.base_arch == 'x86_64':
+            self.debian_arch = 'amd64'
+        elif self.base_arch == 'ppc64le':
+            self.debian_arch = 'ppc64el'
         self.images = list()
         rpm_setup_config = ([repo_file for repo_file in
                              conf.rpm_setup_config if repo_file is not None])
@@ -692,6 +703,35 @@ class KollaWorker(object):
         deb_base = ['ubuntu', 'debian']
         deb_type = ['source', 'binary']
 
+        if self.base in rh_base and self.base_tag.startswith('7'):
+            self.conf.distro_python_version = "2.7"
+        elif self.base in rh_base and self.base_tag.startswith('8'):
+            self.conf.distro_python_version = "3.6"
+        else:
+            # Assume worst
+            self.conf.distro_python_version = "2.7"
+
+        if self.conf.distro_package_manager is not None:
+            package_manager = self.conf.distro_package_manager
+        elif self.base in rh_base:
+            if LooseVersion(self.base_tag) >= LooseVersion('8'):
+                package_manager = 'dnf'
+            else:
+                package_manager = 'yum'
+        elif self.base in deb_base:
+            package_manager = 'apt'
+        self.distro_package_manager = package_manager
+
+        self.clean_package_cache = self.conf.clean_package_cache
+
+        # Determine base packaging type for use in Dockerfiles.
+        if self.conf.base_package_type:
+            self.base_package_type = self.conf.base_package_type
+        elif self.base in rh_base:
+            self.base_package_type = 'rpm'
+        elif self.base in deb_base:
+            self.base_package_type = 'deb'
+
         if not ((self.base in rh_base and self.install_type in rh_type) or
                 (self.base in deb_base and self.install_type in deb_type)):
             raise exception.KollaMismatchBaseTypeException(
@@ -721,14 +761,20 @@ class KollaWorker(object):
         self.image_statuses_unmatched = dict()
         self.image_statuses_skipped = dict()
         self.maintainer = conf.maintainer
+        self.distro_python_version = conf.distro_python_version
 
         docker_kwargs = docker.utils.kwargs_from_env()
         try:
             self.dc = docker.APIClient(version='auto', **docker_kwargs)
         except docker.errors.DockerException as e:
             self.dc = None
-            if not conf.template_only:
-                raise e
+            if not (conf.template_only
+                    or conf.save_dependency
+                    or conf.list_images
+                    or conf.list_dependencies):
+                LOG.error("Unable to connect to Docker, exiting")
+                LOG.info("Exception caught: {0}".format(e))
+                sys.exit(1)
 
     def _get_images_dir(self):
         possible_paths = (
@@ -882,6 +928,8 @@ class KollaWorker(object):
                       'base_distro_tag': self.base_tag,
                       'base_arch': self.base_arch,
                       'use_dumb_init': self.use_dumb_init,
+                      'base_package_type': self.base_package_type,
+                      'debian_arch': self.debian_arch,
                       'supported_distro_release': supported_distro_release,
                       'install_metatype': self.install_metatype,
                       'image_prefix': self.image_prefix,
@@ -892,8 +940,11 @@ class KollaWorker(object):
                       'kolla_version': kolla_version,
                       'image_name': image_name,
                       'users': self.get_users(),
+                      'distro_python_version': self.distro_python_version,
+                      'distro_package_manager': self.distro_package_manager,
                       'rpm_setup': self.rpm_setup,
-                      'build_date': build_date}
+                      'build_date': build_date,
+                      'clean_package_cache': self.clean_package_cache}
             env = jinja2.Environment(  # nosec: not used to render HTML
                 loader=jinja2.FileSystemLoader(self.working_dir))
             env.filters.update(self._get_filters())
